`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
//[2:0]ad7606_os_o ，三个引脚同时为高，开启软件模式，
// 然后分开convst_A和convst_B跳线帽，convst_B作为写使能
// cs和convst_B同时拉低，
// 16位数据需要在convst_B拉高前5ns准备好
//  16位数据需要在convst_B拉高后继续保持5ns
// 写周期最低180ns
// convst_B高电平2ns，低电平35ns
// 指令组成，高八位是地址，低八位是数据
///读取操作，先进入写模式，然后数据中最高位必须为1，紧跟着的7位代表地址，低八位不关心
// 然后写信号拉高，读信号拉低，开始读取数据    0+地址+数据
//恢复正常数据转换，保持全低180ns。
//////////////////////////////////////////////////////////////////////////////////


module ad7606C_soft_driver(
	Clk,
	Reset_n,
	Go,
	//////
	ADC_soft_modle,
	DataNum,
	RestartReq,
	Speed_Set,
	////////
	ad7606_cs_n_o,   
	ad7606_rd_n_o, 
	ad7606_wr_n_o,	
	ad7606_busy_i,   
	ad7606_db_i,     
	ad7606_os_o,     
	ad7606_reset_o,  
	ad7606_convst_o,
	Conv_Done, 
	
	data_flag,
	data_mult_ch,
	data1,
	data2, 
	data3, 
	data4, 
	data5, 
	data6, 
	data7, 
	data8
);


	input  wire 		 Clk;
	input  wire 		 Reset_n;
	input  wire 		 Go;
	input                RestartReq;  // 采样起始信号
	input [24:0]		 Speed_Set;   // 采样速度设置
	input [31:0]		 ADC_soft_modle; // 软件模式下，修改寄存器指令
	input [31:0]		 DataNum;        // 采样数量
	
	output reg  		 Conv_Done;
	output reg        ad7606_cs_n_o;         
	output reg         ad7606_rd_n_o;        
	output reg         ad7606_wr_n_o;   // 软件模式，写使能信号，      
	input  wire        ad7606_busy_i;         
	inout  wire [15:0] ad7606_db_i;    // 软件模式下需要输出，所以这里就是 inout       
	output reg [2:0]  ad7606_os_o;           
	output reg         ad7606_reset_o;        
	
	output reg         ad7606_convst_o;       
	output reg [7:0]data_flag;
	output reg [15:0]data_mult_ch;
	output reg [15:0]data1,data2, data3, data4, data5, data6, data7, data8;
	
	
	reg [6:0]state;
	reg [1:0]ad7606_busy_r;

	reg [24:0]cnt;
	reg [5:0] cnt_reg;         // 软件模式下，写寄存器等待计数器
	reg [31:0]	cnt_DataNum;   // 计数一个指令所需要的数据，用于判断新的模式选择，
	wire trig;	
	
    reg         write_flag;   // 三态控制信号
    reg [15:0] data_reg_i;  // 接收读回寄存器值
    reg [15:0] data_reg_o; // 待写入寄存器值	
  

	always@(posedge Clk)
		ad7606_busy_r <= {ad7606_busy_r[0],ad7606_busy_i};
	
	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)
		data_flag <= 0;
	else begin
		data_flag[0] <= state == 26;
		data_flag[1] <= state == 29;
		data_flag[2] <= state == 32;
		data_flag[3] <= state == 35;
		data_flag[4] <= state == 38;
		data_flag[5] <= state == 41;
		data_flag[6] <= state == 44;
		data_flag[7] <= state == 47;	
	end
	
	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)
		data_mult_ch <= 0;
	else begin
		data_mult_ch <= 
			 (  (state == 26)
			 || (state == 29)
			 || (state == 32)
			 || (state == 35)
			 || (state == 38)
			 || (state == 41)
			 || (state == 44)
			 || (state == 47)
			)? ad7606_db_i:data_mult_ch;	
	end


	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)
		cnt <= 0;
	else if(cnt >= Speed_Set)
		cnt <= 0;
	else
		cnt <= cnt + 1'b1;
		
	assign trig = cnt == Speed_Set;

	
    assign ad7606_db_i = write_flag ? data_reg_o:16'hzzzz;
    
    reg [15:0] lut;     // 寄存器表
    reg [5:0] cnt_table; // 寄存器表地址
    
    reg [31:0] cnt_reset;   // 大复位下，拉高时间计数器
    reg [31:0] cnt_setup;   // 大复位下，拉高后释放等待计数器
    parameter full_reset = 320;     // t_RESET >= 3200ns        全局复位，大复位
    parameter reset_setup = 27400;  //t_DEVICE_SETUP >= 274 us

always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)begin
		state <= 0;
		ad7606_convst_o <= 1;
		Conv_Done <= 0;
		data1 <= 0; data2 <= 0; data3 <= 0; data4 <= 0;
		data5 <= 0; data6 <= 0; data7 <= 0; data8 <= 0;
		ad7606_rd_n_o <= 1;
		ad7606_reset_o <= 0;
		
		cnt_reg <= 6'd0;
		ad7606_cs_n_o <= 1'b1;
		ad7606_wr_n_o <= 1'b1;
		data_reg_i <= 16'd0;
		data_reg_o <= 16'd0;
		write_flag <= 1'b0;
		ad7606_os_o = 3'b000;
		cnt_DataNum <= 32'd0;
		
		cnt_table <= 6'd0;
		cnt_reset <= 32'd0;
		cnt_setup <= 32'd0;		
    end	else begin
        case(state)
            00:begin
                cnt_DataNum <= 32'd0;
                ad7606_cs_n_o <= 1'b0;
                if (RestartReq) begin
                    if (ADC_soft_modle[15:0] == 16'd1) begin /// 进入软件模式，初始化寄存器表
                        state <= state + 1'b1;
                        ad7606_os_o = 3'b111;
                    end else if (ADC_soft_modle != 32'd0) begin // 进入软件模式，修改单个寄存器
                        state <= 10;
                        ad7606_os_o = 3'b111;
                    end else begin                              // 进入硬件模式，开始采集
                        state <= 20;
                        ad7606_os_o = 3'b000;
                    end
                end else begin
                     state <=  state;
                end
            end
/////////////////////////////////////
            01:begin  //全局大复位，清除寄存器配置，然后重新配置寄存器
		          cnt_reset <= cnt_reset + 1'b1;
		          ad7606_reset_o <= 1'b1;
		          ad7606_os_o = 3'b111;
		          if (cnt_reset >= full_reset) begin
		              state <= state + 1'b1;
                  end else begin
                    state <= state;
                  end
		      end
		      02:begin
		          cnt_setup <= cnt_setup + 1'b1;
		          ad7606_reset_o <= 1'b0;
		          if (cnt_setup >= reset_setup) begin
		              state <= state + 1'b1;
                  end else begin
                    state <= state;
                  end
		      end
/////////////////////////////////////////////////        
            03:begin                            // 开始写寄存器表，初始化寄存器
                if (cnt_table < 6'd40) begin
                    write_flag <= 1'b1; 
                    ad7606_wr_n_o <= 1'b0; 
                    cnt_reg <= 5'd0;  
                    state <= state + 1'b1;
                end else begin
                    state <= 13;        // 写寄存器表完成，然后进入状态13，软硬件模式转换
                end
            end
            04:begin                            // 等待写使能拉低40ns，然后准备好带写入的值
                data_reg_o <= {1'b0,lut[14:0]};
                if (cnt_reg >= 5'd3) begin
                    state <= state + 1'b1; 
                    cnt_reg <= 5'd0;
                end else begin
                    cnt_reg <= cnt_reg + 1'b1; 
                    state <= state; 
                end
            end
            05:begin                        // 开始写寄存操作，
                ad7606_wr_n_o <= 1'b1;
                if (cnt_reg >= 5'd14) begin         //等待一个周期180ns，
                    state <= 03;                    // 然后回到写寄存器表状态，继续下一个寄存器
                    cnt_reg <= 5'd0; 
                    cnt_table <= cnt_table + 1'b1; // 寄存器表地址加一
                end else begin
                    cnt_reg <= cnt_reg + 1'b1; 
                    state <= state; 
                end
            end
//////////////////////////////////////////////////////////            
            10:begin                    // 指令修改单个寄存器
                cnt_DataNum <= 32'd0;
                cnt_reg <= 5'd0;
                write_flag <= 1'b1;
                ad7606_wr_n_o <= 1'b0;
                state <= state + 1'b1;
            end
            11:begin                            // 等待写使能拉低40ns，然后准备好带写入的值
                data_reg_o <= {1'b0,ADC_soft_modle[30:16]};
                if (cnt_reg >= 5'd3) begin
                    state <= state + 1'b1; 
                    cnt_reg <= 5'd0;
                end else begin
                    cnt_reg <= cnt_reg + 1'b1; 
                    state <= state; 
                end
            end
            12:begin                        // 开始写寄存操作，
                ad7606_wr_n_o <= 1'b1;
                if (cnt_reg >= 5'd14) begin         //等待一个周期180ns，
                    state <= state + 1'b1;          // 写操作完成，
                    cnt_reg <= 5'd0; 
                end else begin
                    cnt_reg <= cnt_reg + 1'b1; 
                    state <= state; 
                end
            end
            13:begin                    // 退出软件模式，进入硬件模式，
                write_flag <= 1'b0;     // 释放flga信号，只能输入
                ad7606_wr_n_o <= 1'b1;
                ad7606_os_o = 3'b000;
                state <= 20;
            end
////////////////////////////////////////// 正常的硬加模式下，数据采集转换
            20: begin
				if(Go && trig)begin
					state <= state + 1'b1;
					ad7606_convst_o <= 0;
					ad7606_rd_n_o <= 1;
					Conv_Done <= 0;
					ad7606_reset_o <= 0;
				end
				else begin
					state <= state;
					ad7606_convst_o <= 1;
				end
            end
			21: begin state <= state + 1'b1; ad7606_convst_o <= 1;end
			22: if(ad7606_busy_r==2'b10)begin  /// 等待芯片转换完成，
                    state <= state + 1'b1;
                    ad7606_rd_n_o <= 0;
			     end else  begin
			         state <= state;
                 end
			23: state <= state + 1'b1;
////////////////////////////////////////////////////////////////////////////	
			24: begin ad7606_rd_n_o <= 1; state <= state + 1'b1;end
			25: begin  state <= state + 1'b1;end
			26: begin state <= state + 1'b1; ad7606_rd_n_o <= 0; data1 <=  ad7606_db_i;end // 通道1
////////////////////////////////////////////////////////////////////////////
			27: begin ad7606_rd_n_o <= 1;  state <= state + 1'b1;end
			28: begin  state <= state + 1'b1;end
			29: begin state <= state + 1'b1;ad7606_rd_n_o <= 0; data2 <= ad7606_db_i;end // 通道2
////////////////////////////////////////////////////////////////////////////
			30: begin ad7606_rd_n_o <= 1;  state <= state + 1'b1;end	
			31: begin  state <= state + 1'b1;end
			32: begin state <= state + 1'b1;ad7606_rd_n_o <= 0; data3 <= ad7606_db_i;end // 通道3
////////////////////////////////////////////////////////////////////////////
			33: begin ad7606_rd_n_o <= 1;  state <= state + 1'b1;end		
			34: begin  state <= state + 1'b1;end
			35: begin state <= state + 1'b1;ad7606_rd_n_o <= 0; data4 <=  ad7606_db_i;end // 通道4
////////////////////////////////////////////////////////////////////////////
			36: begin ad7606_rd_n_o <= 1;  state <= state + 1'b1;end
			37: begin  state <= state + 1'b1;end
			38: begin state <= state + 1'b1;ad7606_rd_n_o <= 0; data5 <=  ad7606_db_i;end // 通道5
////////////////////////////////////////////////////////////////////////////
			39: begin ad7606_rd_n_o <= 1;  state <= state + 1'b1;end
			40: begin state <= state + 1'b1;end
			41: begin state <= state + 1'b1;ad7606_rd_n_o <= 0;data6 <=  ad7606_db_i; end // 通道6
////////////////////////////////////////////////////////////////////////////
			42: begin ad7606_rd_n_o <= 1; state <= state + 1'b1;end
			43: begin  state <= state + 1'b1; end
			44: begin state <= state + 1'b1;ad7606_rd_n_o <= 0; data7 <=  ad7606_db_i; end  // 通道7
////////////////////////////////////////////////////////////////////////////
			45: begin ad7606_rd_n_o <= 1;  state <= state + 1'b1;end	
			46: begin  state <= state + 1'b1;end	
			47: begin state <= state + 1'b1; ad7606_reset_o <= 1; data8 <=  ad7606_db_i;end // 通道8
////////////////////////////////////////////////////////////////////////////
			48: begin state <= state + 1'b1; Conv_Done <= 1;end
			49: begin state <= state + 1'b1; Conv_Done <= 0;end
			50: begin state <= state + 1'b1;  end 
			51: begin state <= state + 1'b1;  end 
			52: begin state <= state + 1'b1;  end 
			53: begin state <= state + 1'b1;  ad7606_reset_o <= 0;end 
			54: begin state <= state + 1'b1;  end 
			55: begin state <= state + 1'b1; cnt_DataNum <= cnt_DataNum + 1'b1; end 
			56: begin 
			     if (cnt_DataNum >= DataNum) begin  // 判断采集转换的数量是否完成指令所需
			         state <= 00;              // 然后跳回到00状态，等待新的指令，硬件？寄存器表？单个寄存器
			        cnt_table <= 6'd0;
                    cnt_reset <= 32'd0;
                    cnt_setup <= 32'd0;
                 end  else begin
			         state <= 20;
                 end
             end 
			default: begin
					state <= 00;
					ad7606_convst_o <= 1;
					Conv_Done <= 0;
					data1 <= 0; data2 <= 0; data3 <= 0; data4 <= 0;
					data5 <= 0; data6 <= 0; data7 <= 0; data8 <= 0;
					ad7606_rd_n_o <= 1;
					ad7606_reset_o <= 0;
					cnt_reg <= 6'd0;
                    ad7606_wr_n_o <= 1'b1;
                    data_reg_i <= 16'd0;
                    data_reg_o <= 16'd0;
                    ad7606_cs_n_o <= 1'b1;
                    write_flag <= 1'b0;
                    ad7606_os_o = 3'b000;
                    cnt_DataNum <= 32'd0;
                    cnt_table <= 6'd0;
                    cnt_reset <= 32'd0;
                    cnt_setup <= 32'd0;	
            end
        endcase        
    end
	
////////////////////////////////////   寄存器表   //////////////////////////
  reg [15:0] rom [39:0];

  initial begin
    rom[00] = 16'h02_08; // STATUS  了解设备的配置状态，包括操作模式、状态头是否启用、是否使用外部过采样时钟等。
    // 用于设置和读取通道1-8的模拟输入范围
    rom[01] = 16'h03_11;  // RANGE_CH1_CH2  0001: ±5 V  0000: ±2.5 V 
    rom[02] = 16'h04_11;  // RANGE_CH3_CH4  0001: ±5 V  0000: ±2.5 V 
    rom[03] = 16'h05_11;  // RANGE_CH5_CH6  0001: ±5 V  0000: ±2.5 V 
    rom[04] = 16'h06_11;  // RANGE_CH7_CH8  0001: ±5 V  0000: ±2.5 V 
    
    rom[05] = 16'h07_00;  // 通道的带宽配置
    rom[06] = 16'h08_00;  // 过采样比率和过采样填充 00-off 0001-2 0010-4 0011-8
    //通道的增益校准。
    rom[07] = 16'h09_00;  //  CH1_GAIN. Resolution: 1024 ?. Range: 0 ? to 65,536 ?.
    rom[08] = 16'h0a_00;  //  CH2_GAIN. Resolution: 1024 ?. Range: 0 ? to 65,536 ?.
    rom[09] = 16'h0b_00;  //  CH3_GAIN. Resolution: 1024 ?. Range: 0 ? to 65,536 ?.
    rom[10] = 16'h0c_00;  //  CH4_GAIN. Resolution: 1024 ?. Range: 0 ? to 65,536 ?.
    rom[11] = 16'h0d_00;  //  CH5_GAIN. Resolution: 1024 ?. Range: 0 ? to 65,536 ?.
    rom[12] = 16'h0e_00;  //  CH6_GAIN. Resolution: 1024 ?. Range: 0 ? to 65,536 ?.
    rom[13] = 16'h0f_00;  //  CH7_GAIN. Resolution: 1024 ?. Range: 0 ? to 65,536 ?.
    rom[14] = 16'h10_00;  //  CH8_GAIN. Resolution: 1024 ?. Range: 0 ? to 65,536 ?.
    //通道的偏移校准。
    rom[15] = 16'h11_80;  //  CH1_OFFSET. Range from -512 LSB to +511 LSB.
    rom[16] = 16'h12_80;  //  CH2_OFFSET. Range from -512 LSB to +511 LSB.
    rom[17] = 16'h13_80;  //  CH3_OFFSET. Range from -512 LSB to +511 LSB.
    rom[18] = 16'h14_80;  //  CH4_OFFSET. Range from -512 LSB to +511 LSB.
    rom[19] = 16'h15_80;  //  CH5_OFFSET. Range from -512 LSB to +511 LSB.
    rom[20] = 16'h16_80;  //  CH6_OFFSET. Range from -512 LSB to +511 LSB.
    rom[21] = 16'h17_80;  //  CH7_OFFSET. Range from -512 LSB to +511 LSB.
    rom[22] = 16'h18_80;  //  CH8_OFFSET. Range from -512 LSB to +511 LSB.
    //通道的相位校准。
    rom[23] = 16'h19_00;  //  CH1_PHASE. . Phase delay from 0 ?s to 255 ?s in steps of 1 ?s.
    rom[24] = 16'h1a_00;  //  CH2_PHASE. . Phase delay from 0 ?s to 255 ?s in steps of 1 ?s.
    rom[25] = 16'h1b_00;  //  CH3_PHASE. . Phase delay from 0 ?s to 255 ?s in steps of 1 ?s.
    rom[26] = 16'h1c_00;  //  CH4_PHASE. . Phase delay from 0 ?s to 255 ?s in steps of 1 ?s.
    rom[27] = 16'h1d_00;  //  CH5_PHASE. . Phase delay from 0 ?s to 255 ?s in steps of 1 ?s.
    rom[28] = 16'h1e_00;  //  CH6_PHASE. . Phase delay from 0 ?s to 255 ?s in steps of 1 ?s.
    rom[29] = 16'h1f_00;  //  CH7_PHASE. . Phase delay from 0 ?s to 255 ?s in steps of 1 ?s.
    rom[30] = 16'h20_00;  //  CH8_PHASE. . Phase delay from 0 ?s to 255 ?s in steps of 1 ?s.
    
    rom[31] = 16'h21_01;  //  DIGITAL_DIAG_ENABLE
    rom[32] = 16'h22_00;  //  DIGITAL_DIAG_ERR
    rom[33] = 16'h23_00;  //  OPEN_DETECT_ENABLE
    rom[34] = 16'h24_00;  //  OPEN_DETECTED
    
    // 诊断多路复用器的配置，可以用于监测内部节点。
    rom[35] = 16'h28_00;  //  DIAGNOSTIC_MUX_CH1_2
    rom[36] = 16'h29_00;  //  DIAGNOSTIC_MUX_CH3_4
    rom[37] = 16'h2a_00;  //  DIAGNOSTIC_MUX_CH5_6
    rom[38] = 16'h2b_00;  //  DIAGNOSTIC_MUX_CH7_8
    
    rom[39] = 16'h2c_00;  //  开路检测队列，配置自动或手动开路检测。1手动
  end

  always @ (posedge Clk)
  begin
    lut <= rom[cnt_table];
  end
	
	
endmodule
