module ad7606C_udp_rgmii(
	inclk0				,
	rst_n				,
	//ACM7606模块接口信号
	ad7606_cs_n_o	,
	ad7606_rd_n_o	, 
	ad7606_wr_n_o	,
	ad7606_os_o		,
	ad7606_convst_o,
	ad7606_db_i		,
	ad7606_reset_o	,
	ad7606_busy_i	,

	//eth_reset
	eth_reset		,
	mdc				,
	mdio				,
	//eth_rx
	rgmii_rx_clk_i	,
	rgmii_rxd		,
	rgmii_rxdv		,
	
	//eth_tx			
	rgmii_tx_clk	,
	rgmii_txd		,
	rgmii_txen		
	
);

	input				inclk0				;
	input				rst_n				;
	
	//ad7606
	output        	ad7606_cs_n_o	; 
	output        	ad7606_rd_n_o	;         
	output 	      ad7606_wr_n_o	; 
	input 			ad7606_busy_i	;
	inout  [15:0]  ad7606_db_i		;
	output [2:0]  	ad7606_os_o		; 
	output        	ad7606_reset_o	;
	output        	ad7606_convst_o;

	
	output			eth_reset		;
	output 			mdc				;
	inout				mdio				;
	//eth_rx
	input				rgmii_rx_clk_i	;
	input  	[3:0] rgmii_rxd		;
	input				rgmii_rxdv		;
	
	//eth_tx			
	output			rgmii_tx_clk	;
	output	[3:0]	rgmii_txd		;
	output			rgmii_txen		;
	

	parameter local_mac = 48'h00_0a_35_01_fe_c0;
//	parameter local_mac = 48'h00_07_ed_ac_62_00;
	parameter local_ip = 32'hc0_a8_00_02;
	parameter local_port = 16'd5000;
  
	wire phy_init;
  
	phy_config	phy_reg_config(
		.clk(clk),
		.rst_n(rst_n),
		.phy_rst_n(eth_reset),
		.phy_init(phy_init),
		.mdc(mdc),
		.mdio(mdio)
	);

	wire rgmii_rx_clk;
	 //调相
	rx_pll pll_125m(
		.inclk0(rgmii_rx_clk_i),
		.c0(rgmii_rx_clk)
	);

	wire clk;
	 //调相
	PLL_100 PLL_100(
		.inclk0(inclk0),
		.c0(clk)
	);
	
	wire gmii_rx_clk;
	wire [7:0] gmii_rxd;
	wire gmii_rxdv;
	
	rgmii_to_gmii	rgmii_to_gmii(
		.rgmii_rxc	(rgmii_rx_clk),
		.rgmii_rxd	(rgmii_rxd),
		.rgmii_rxdv	(rgmii_rxdv),
		.gmii_rxc	(gmii_rx_clk),
		.gmii_rxd	(gmii_rxd),
		.gmii_rxdv  (gmii_rxdv)
	);
	
	wire clk125m_o;
   wire [7:0]payload_dat_o;
   wire payload_valid_o;
   wire one_pkt_done;

	eth_udp_rx_gmii eth_udp_rx_gmii (
	  .reset_n (rst_n),
	  
	  .gmii_rx_clk(gmii_rx_clk),  
	  .gmii_rxdv(gmii_rxdv),
	  .gmii_rxd(gmii_rxd),
	  //out clk
	  .clk125m_o(clk125m_o),
	 
	  .local_mac(local_mac),
	  .local_ip(local_ip),
	  .local_port(local_port),
	  
	  .exter_mac(),
	  .exter_ip(),
	  .exter_port(),
	  
	  .rx_data_length(),
	  .data_overflow_i(),
	  
	  .payload_valid_o(payload_valid_o),
	  .payload_dat_o(payload_dat_o),
	  
	  .one_pkt_done(one_pkt_done),
	  .pkt_error(),
	  .debug_crc_check()   
	); 
	
    wire [7:0]rxdout;
    wire rx_empty;
    wire fifo_rd_req;

	fifo_rx 	fifo_rx(
		.data		(payload_dat_o),
		.rdclk	(clk),
		.rdreq	(fifo_rd_req),
		.wrclk	(clk125m_o),
		.wrreq	(payload_valid_o),
		.q			(rxdout),
		.rdempty	(rx_empty),
		.wrfull	()
	);
	
	 wire cmdvalid_0;
    wire [7:0]address_0;
    wire [31:0]cmd_data_0; 

    eth_cmd eth_cmd (
      .clk(clk),
      .reset_n(rst_n),
      .fifo_rd_req(fifo_rd_req),
      .rx_empty(rx_empty),
      .fifodout(rxdout),
      .cmdvalid(cmdvalid_0),
      .address(address_0),
      .cmd_data(cmd_data_0)
    );

	 
	 wire [7:0]ChannelSel;
    wire [31:0]DataNum;
    wire [31:0]ADC_Speed_Set;   
    wire [31:0]ADC_soft_modle;
    wire RestartReq;
	 
	 cmd_rx cmd_rx_0(
		.clk(clk),
		.reset_n(rst_n),
		.cmdvalid(cmdvalid_0),
		.cmd_addr(address_0),
		.cmd_data(cmd_data_0),
		
		.ChannelSel(ChannelSel),
		.DataNum(DataNum),
		.ADC_Speed_Set(ADC_Speed_Set),
		.ADC_soft_modle(ADC_soft_modle),
		.RestartReq(RestartReq)
	);
	
	 reg RestartReq_0_d0,RestartReq_0_d1;
	 reg [31:0]Number_d0,Number_d1;
	 
	 always@(posedge clk125m_o)
	 begin
		Number_d0 <= DataNum;
		Number_d1 <= Number_d0;
		
		RestartReq_0_d0 <= RestartReq;
		RestartReq_0_d1 <= RestartReq_0_d0;
	 end
	
	 wire [15:0] adc_data_mult_ch;
	 wire[7:0]adc_data_flag;

	//AD7606控制器驱动模块 
    ad7606C_soft_driver ad7606C_soft_driver(
        .Clk(clk),
        .Reset_n(rst_n),
        .Go(1),
        .Speed_Set(ADC_Speed_Set),
        .ADC_soft_modle(ADC_soft_modle),
        .RestartReq(RestartReq),
        .DataNum(DataNum),
        .Conv_Done(),
        .ad7606_cs_n_o(ad7606_cs_n_o), 
        .ad7606_rd_n_o(ad7606_rd_n_o), 
        .ad7606_wr_n_o(ad7606_wr_n_o), 
        .ad7606_busy_i(ad7606_busy_i), 
        .ad7606_db_i(ad7606_db_i), 
        .ad7606_os_o(ad7606_os_o), 
        .ad7606_reset_o(ad7606_reset_o), 
        .ad7606_convst_o(ad7606_convst_o), 
			.data_flag(adc_data_flag),
			.data_mult_ch(adc_data_mult_ch),
        .data1(),
        .data2(), 
        .data3(), 
        .data4(), 
        .data5(), 
        .data6(), 
        .data7(), 
        .data8()
    );	
	 
//	ad7606C_driver		ad7606C_driver(
//		.Clk					(clk),
////		.Rst_n				(rst_n),
//		.Reset_n				(rst_n),
//		.Go					(1),
//		.Speed_Set			(ADC_Speed_Set),
//		.Conv_Done			(Conv_Done),
//		.trig					(trig),
//		.convst_o			(convst_o),	
//		.ad7606_cs_n_o		(ad7606_cs_n_o),   
//		.ad7606_rd_n_o		(ad7606_rd_n_o),   
//		.ad7606_busy_i		(ad7606_busy_i),   
//		.ad7606_db_i		(ad7606_db_i), 		    
//		.ad7606_os_o		(ad7606_os_o),     
//		.ad7606_reset_o	(ad7606_reset_o),  
//		.ad7606_convst_o	(ad7606_convst_o), 
//		.data_flag			(adc_data_flag),
//		.data_mult_ch		(adc_data_mult_ch),
//		.data1				(),
//		.data2				(), 
//		.data3				(), 
//		.data4				(), 
//		.data5				(), 
//		.data6				(), 
//		.data7				(), 
//		.data8				()
//	);
	
	 wire fifowrreq;   
	 wire [15:0]ad_out;
    adc_write_ctrl adc_write_ctrl(
        .Clk(clk),
        .Reset_n(rst_n),
        .DataNum(DataNum),
        .RestartReq(RestartReq),
        .ChannelSel(ChannelSel),
        .fifowrreq(fifowrreq),
        .fifowrdata(ad_out),
        .fifowrfull(),
        .adc_data_flag(adc_data_flag),
        .adc_data_mult_ch(adc_data_mult_ch)
    );
	 
	 wire payload_req_o;
	 wire [7:0]dout;
	 wire txempty;
	 wire [14:0]rd_data_count;
    wire [13:0]wr_data_count;
	 
	fifo_tx fifo_tx(
		.data				(ad_out),
		.rdclk			(clk125m_o),
		.rdreq			(payload_req_o),
		.wrclk			(clk),
		.wrreq			(fifowrreq),
		.q					(dout),
		.rdempty			(txempty),
		.rdusedw			(rd_data_count),
		.wrfull			(),
		.wrusedw			(wr_data_count)
	);

	wire tx_done;
	wire tx_en_pulse;
	wire [15:0]lenth_val;
	
    eth_send_ctrl eth_send_ctrl(
      .clk125M(clk125m_o),     
      .rst_n	(rst_n),  
      .eth_tx_done(tx_done),  
      .restart_req(RestartReq_0_d1),
      .fifo_rd_cnt(rd_data_count),
      .total_data_num(Number_d1),  
      .pkt_tx_en(tx_en_pulse),  
      .pkt_length(lenth_val) 
   ); 
	
	 wire gmii_tx_clk;
    wire [7:0]gmii_txd;
    wire gmii_txen;
	
   eth_udp_tx_gmii eth_udp_tx_gmii
    (
        .clk125M       (clk125m_o             ),
        .reset_n        (rst_n                 ),

        .tx_en_pulse   (tx_en_pulse           ),
        .tx_done       (tx_done               ),

        .dst_mac       (48'hFF_FF_FF_FF_FF_FF),
        .src_mac       (48'h00_0a_35_01_fe_c0), 
        .dst_ip        (32'hc0_a8_00_03      ),
        .src_ip        (32'hc0_a8_00_02      ),
        .dst_port      (16'd6102             ),
        .src_port      (16'd5000              ),
        

        .data_length   (lenth_val            ),
        
        .payload_req_o (payload_req_o        ),
        .payload_dat_i (dout                 ),

        .gmii_tx_clk   (gmii_tx_clk          ),
        .gmii_txen     (gmii_txen            ),
        .gmii_txd      (gmii_txd             )
    );

	 
	gmii_to_rgmii	gmii_to_rgmii(
		.gmii_gtxc(gmii_tx_clk),
		.gmii_txd(gmii_txd),
		.gmii_txen(gmii_txen),
		.rgmii_gtxc(rgmii_tx_clk),
		.rgmii_txd(rgmii_txd),
		.rgmii_txen(rgmii_txen)
	);

endmodule 