module eth_receive_cmd#(
    parameter local_mac = 48'h00_07_ed_ac_62_00,
    parameter local_ip = 32'hc0_a8_00_02,
    parameter local_port = 16'd5000
)
(
	 rgmii_rx_clk_i	,
    clk_50M				,
    rst_n				,
    rgmii_rxd			,
    rgmii_rxdv			,
    clk125m_o			,
    ChannelSel			,
    DataNum				,
    ADC_Speed_Set		,
    RestartReq			
);

    input rgmii_rx_clk_i;
    input clk_50M;
    input rst_n;
    input  [3:0] rgmii_rxd;
    input rgmii_rxdv;

    output clk125m_o;
    output [7:0]ChannelSel;
    output [31:0]DataNum;
    output [31:0]ADC_Speed_Set;
    output RestartReq;

    wire rgmii_rx_clk;
    wire gmii_rx_clk;
    wire [7:0] gmii_rxd;
    wire gmii_rxdv;

    wire [7:0]payload_dat_o;
    wire payload_valid_o;
    wire one_pkt_done;

    //rxfifo 
    wire [7:0]rxdout;
    wire rx_empty;
    wire fifo_rd_req;

    //rxcmd
    wire cmdvalid_0;
    wire [7:0]address_0;
    wire [31:0]cmd_data_0; 

	 
	 //调相
	rx_pll pll_125m(
		.inclk0(rgmii_rx_clk_i),
		.c0(rgmii_rx_clk)
	);
	 
	rgmii_to_gmii	rgmii_to_gmii(
		.rgmii_rxc	(rgmii_rx_clk),
		.rgmii_rxd	(rgmii_rxd),
		.rgmii_rxdv	(rgmii_rxdv),
		.gmii_rxc	(gmii_rx_clk),
		.gmii_rxd	(gmii_rxd),
		.gmii_rxdv  (gmii_rxdv)
	);
	
	eth_udp_rx_gmii eth_udp_rx_gmii (
	  .reset_n (rst_n),
	  
	  .gmii_rx_clk(gmii_rx_clk),  
	  .gmii_rxdv(gmii_rxdv),
	  .gmii_rxd(gmii_rxd),
	  //out clk
	  .clk125m_o(clk125m_o),
	 
	  .local_mac(local_mac),
	  .local_ip(local_ip),
	  .local_port(local_port),
	  
	  .exter_mac(),
	  .exter_ip(),
	  .exter_port(),
	  
	  .rx_data_length(),
	  .data_overflow_i(),
	  
	  .payload_valid_o(payload_valid_o),
	  .payload_dat_o(payload_dat_o),
	  
	  .one_pkt_done(one_pkt_done),
	  .pkt_error(),
	  .debug_crc_check()   
	); 
	
	fifo_rx 	fifo_rx(
		.data		(payload_dat_o),
		.rdclk	(clk_50M),
		.rdreq	(fifo_rd_req),
		.wrclk	(clk125m_o),
		.wrreq	(payload_valid_o),
		.q			(rxdout),
		.rdempty	(rx_empty),
		.wrfull	()
	);
	
    eth_cmd eth_cmd (
      .clk(clk_50M),
      .reset_n(rst_n),
      .fifo_rd_req(fifo_rd_req),
      .rx_empty(rx_empty),
      .fifodout(rxdout),
      .cmdvalid(cmdvalid_0),
      .address(address_0),
      .cmd_data(cmd_data_0)
    );
   
   cmd_rx cmd_rx_0(
		.clk(clk_50M),
		.reset_n(rst_n),
		.cmdvalid(cmdvalid_0),
		.cmd_addr(address_0),
		.cmd_data(cmd_data_0),
		
		.ChannelSel(ChannelSel),
		.DataNum(DataNum),
		.ADC_Speed_Set(ADC_Speed_Set),
		.RestartReq(RestartReq)
	);



endmodule 