module eth_send_ctrl(
	clk125M			,
	rst_n				,
	eth_tx_done		,	//以太网一个包发送完毕信号
	restart_req		,
	fifo_rd_cnt		,	//从FIFO中读取的数据个数
	total_data_num	,	//需要发送的数据总数
	pkt_length		,
	pkt_tx_en		
);

	input clk125M;
	input rst_n;
	input eth_tx_done;
	input restart_req;
	input [10:0]fifo_rd_cnt;
	input [31:0] total_data_num;
	output reg [15:0]pkt_length;
	output reg pkt_tx_en; 

	//采集数据最大字节数：1500-IP报文头部（20字节）-UDP报文头部（8字节）= 1472字节
	reg[3:0]state;
	reg [31:0] data_num;
	reg [28:0]cnt_dly_time;
	
	parameter cnt_dly_min = 16'd128;

	always @ (posedge clk125M or negedge rst_n)
	if (!rst_n)
		begin
			state <= 0;
			cnt_dly_time <= 'd0;
			data_num <= 'd0;
			pkt_tx_en <= 1'b0;
			pkt_length <= 'd0;
		end
	else 
		case (state)
			0	:	begin
						if (restart_req)
							begin
								data_num <= total_data_num;
								if ((total_data_num << 1) >= 'd1472)
									begin
										pkt_length <= 'd1472;
										state <= 1;
									end
								else if ((total_data_num << 1) > 0)
									begin
										pkt_length <= total_data_num << 1;
										state <= 1;
									end
								else
									state <= 0;
							end
					end
					
			1	:	begin
						if (fifo_rd_cnt >= pkt_length - 1)
							begin
								state <= 2;
								pkt_tx_en <= 1'b1;
							end
						else
							begin
								state <= 1;
								pkt_tx_en <= 1'b0;
							end
					end
					
			2	:	begin
						pkt_tx_en <= 1'b0;
						if (eth_tx_done)
							begin
								data_num <= data_num - pkt_length/2;
								state <= 3;
							end
					end
					
			3	:	begin
						if (cnt_dly_time >= cnt_dly_min)
							begin
								state <= 4;
								cnt_dly_time <= 'd0;
							end
						else
							begin
								state <= 3;
								cnt_dly_time <= cnt_dly_time + 1'd1;
							end
					end
					
			4	:	begin
						if (data_num * 2 >= 'd1472)
							begin
								pkt_length <='d1472;
								state <= 1;
							end
						else if (data_num * 2 > 0)
							begin
								pkt_length <= data_num * 2;
								state <= 1;
							end
						else
							state <= 0;
					end
			default : state <= 0;
		endcase

endmodule 
