module eth_send_data#(
    parameter dst_mac = 48'hFF_FF_FF_FF_FF_FF,
    parameter src_mac = 48'h00_07_ed_ac_62_00,
    parameter dst_ip = 32'hc0_a8_00_03,
    parameter src_ip = 32'hc0_a8_00_02,
    parameter dst_port = 16'd6102,
    parameter src_port = 16'd5000
)
(
    input rst_n,
    input clk_50M,
    input clk125m_o,
    input [15:0] eth_fifo_wrdata,
    input eth_fifo_wrreq,
    input RestartReq_0_d1,
    input [31:0]Number_d1,

    output       rgmii_tx_clk,
    output [3:0] rgmii_txd,
    output       rgmii_txen
);
    wire [7:0] dout;
    wire eth_fifo_tx_empty;
    wire [13:0]eth_fifo_usedw;
    wire [14:0]rd_data_count;

    wire tx_done;
    wire tx_en_pulse;
    wire [15:0]lenth_val;
    wire	payload_req_o; 
    
    wire gmii_tx_clk;
    wire [7:0]gmii_txd;
    wire gmii_txen;
	
	fifo_tx fifo_tx(
		.data				({eth_fifo_wrdata[7:0],eth_fifo_wrdata[15:8]}),
		.rdclk			(clk125m_o),
		.rdreq			(payload_req_o),
		.wrclk			(clk_50M),
		.wrreq			(eth_fifo_wrreq),
		.q					(dout),
		.rdempty			(eth_fifo_tx_empty),
		.rdusedw			(rd_data_count),
		.wrfull			(),
		.wrusedw			(eth_fifo_usedw)
	);

    eth_send_ctrl eth_send_ctrl(
      .clk125M(clk125m_o),     
      .rst_n	(rst_n),  
      .eth_tx_done(tx_done),  
      .restart_req(RestartReq_0_d1),
      .fifo_rd_cnt(rd_data_count),
      .total_data_num(Number_d1),  
      .pkt_tx_en(tx_en_pulse),  
      .pkt_length(lenth_val) 
   ); 


    eth_udp_tx_gmii eth_udp_tx_gmii
    (
        .clk125M       (clk125m_o             ),
        .reset_n        (rst_n                 ),

        .tx_en_pulse   (tx_en_pulse           ),
        .tx_done       (tx_done               ),

        .dst_mac       (dst_mac),
        .src_mac       (src_mac), 
        .dst_ip        (dst_ip      ),
        .src_ip        (src_ip      ),
        .dst_port      (dst_port             ),
        .src_port      (src_port             ),
        

        .data_length   (lenth_val            ),
        
        .payload_req_o (payload_req_o        ),
        .payload_dat_i (dout                 ),

        .gmii_tx_clk   (gmii_tx_clk          ),
        .gmii_txen     (gmii_txen            ),
        .gmii_txd      (gmii_txd             )
    );

	 
	gmii_to_rgmii	gmii_to_rgmii(
		.gmii_gtxc(gmii_tx_clk),
		.gmii_txd(gmii_txd),
		.gmii_txen(gmii_txen),
		.rgmii_gtxc(rgmii_tx_clk),
		.rgmii_txd(rgmii_txd),
		.rgmii_txen(rgmii_txen)
	);
endmodule 